\begin{code}
module DT where
import PIL
\end{code}

Dehn-Thurston coordinates for the space of curves on a surface.  To
parametrize a given curve X, the surface is divided into pairs of
pants by a maximal collection of disjoint simple closed curves (the
_cuffs_).  Each cuff has an associated _length_ or _measure_ and a
_twist parameter_.

To compute these parameters, pull X tight so that it intersects each
cuff minimally.  The measure of a cuff is the number of times the cuff
intersects X.  To compute the twist, first notice that X intresected
with a given pair of pants looks like at most 3 families of parallel
strands; call each of these families a _seam_.  The center of each
seam is either between two strands or in the center of a strand,
depending on whether there are an even or odd number of strands in the
seam, respectively.  When we glue two pairs of pants across a cuff,
the centers of the seams do not necessarily match up; the twist
parameter measures the failure to line up in the "natural" way (which
is determined by the fat-graph structure).  Note that the "failure to
line up" can be half of an integer when the center of a seam is in the
center of a strand on one side and between two strands on the other
side.  So that we always work with integers rather than half-integer,
the "twist" as stored in a DTAnnulus is _twice_ this natural twist;
once the measures of each seam are fixed, the twist on each cuff is
required to be even or odd in a slightly complicated way; see, for
instance, the function `verifyParity' in Genus2.lhs.

Note that there will be 1, 2, or 4 centers of seams marked on each
side of each cuff, depending on the pattern of seams on the pair of
pants.

There are more uniform ways to describe all of this that makes it
clear why many formulas work more uniformly with this choice of
parameters, but it is a little harder to understand at first.  See [a
paper to be written].

\begin{code}
data DTAnnulus = DTAnnulus { m, t :: Coord } deriving (Show, Read, Ord)

instance Eq DTAnnulus where
    a == b | m a == 0 && m b == 0 = abs (t a) == abs (t b)
	   | otherwise            = m a == m b && t a == t b
\end{code}

There are two frequent operations on the twist parameters: twisting
along the curve, changing the background curve, and performing a full
twist, not changing the background curve but only the way we mark the
surface.

\begin{code}
twist :: Coord -> DTAnnulus -> DTAnnulus
twist tw a = a { t = t a + tw }

dehnTwist :: Integer -> DTAnnulus -> DTAnnulus
dehnTwist tw a = a { t = t a + tw*m a }
\end{code}

'Neighbors' records the lengths of the four neighbors of a given curve
in the pants decomposition.  By convention, the neighbors are read in
cyclic order in the fat graph structure, starting with two neighbors
on the same side of the trivalent vertex: in 'Neighbors a b c d' the
ends correspond to the following diagram:

	  b         c
       	   \	   /
       	    \  	  /
       	     X---X
       	    /	  \
       	   / 	   \
	  a	    d

\begin{code}
type Neighbors = (Coord, Coord, Coord, Coord)
\end{code}

There are a couple of very frequently used utility functions.

lk a b c: the number of strands running from a to b on a pair of pants
with boundary a, b, c

\begin{code}
lk a b c = 0 `up` ((a+b-c)//2) `dn` a `dn` b
\end{code}

ll a b c: the number of strands running from a to a, as above

\begin{code}
ll a b c = ((a-b-c)//2) `up` 0
\end{code}


Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
