\begin{code}
module Genus2IE where
import Genus2
import Genus2Simplify(goodBarbell)
import Interval
import IntervalExchange
import DT
import Monoid
import FreeGroup
import Box
import PIL
import Monad(guard)
import List(all)
\end{code}

Here we convert a curve on a genus 2 surface, represented in
Dehn-Thurston coordinates, into an interval exchange map.

Initially we construct an interval exchange that could be used to
compute the fundamental group.

\begin{code}
gen2ToIE :: Genus2 -> IntervalExchange FreeGroup
gen2ToIE g@(Barbell a b c) | goodBarbell g =
    let [ia, ia', ib, ib', ic, ic'] =
	   makeIntervals (Interval 0 (2*(m a+m b+m c)))
			 [m a,m a,m b,m b,m c,m c]
	v1 = triConnect ia ia' ib
	v2 = triConnect ic ic' ib'
	ca = cyclicConnect ia ia' (t a // 2)
	cb = cyclicConnect ib ib' (t b // 2)
	cc = cyclicConnect ic ic' (t c // 2)
    in map (setData fgid) (v1++v2++cb) ++
       map (setData fga) ca ++
       map (setData fgb) cc
gen2ToIE g | otherwise = error "gen2toIE: input curve not in standard form"
\end{code}

We are interested in connected curves.

\begin{code}
ieConnected :: IntervalExchange a -> Bool
ieConnected ie = case runIE (mapIE (const ()) ie) of
		   [(1,_)] -> True
		   _       -> False
\end{code}

gen2ToIE produces an interval exchange that would compute the
corresponding word in the fundamental group.  For a (considerable)
gain in efficiency, we construct not this word, but instead apply the
check for fibering directly.

\begin{code}
mapIE :: (a -> b) -> IntervalExchange a -> IntervalExchange b
mapIE f = map (fmap f)
\end{code}

First, we compute the homology.

\begin{code}
type Homology = (Integer, Integer)

fggToHomology :: FreeGroupGen -> Homology
fggToHomology (FGPos 0) = (1,0)
fggToHomology (FGNeg 0) = (-1,0)
fggToHomology (FGPos 1) = (0,1)
fggToHomology (FGNeg 1) = (0,-1)

fgToHomology :: FreeGroup -> Homology
fgToHomology (FG l) = foldr (&) unit (map fggToHomology l)
\end{code}

Then, after computing the homology, we project out this element in
homology and consider the resulting walk on the line, using the "Box"
monoid.

\begin{code}
rotate90 :: Homology -> Homology
rotate90 (a,b) = (-b,a)

fggToBox :: Homology -> FreeGroupGen -> Box
-- Special case: if one of the generators maps to 0, the map to boxes
-- needs to be different.
fggToBox (0,b) fgg | gen fgg == 0 = primFlatBox
		   | gen fgg == 1 = primShiftBox (fgsign fgg * b)
fggToBox (a,0) fgg | gen fgg == 0 = primShiftBox (fgsign fgg * a)
		   | gen fgg == 1 = primFlatBox
fggToBox (a,b) fgg | gen fgg == 0 = primBox (fgsign fgg * a)
		   | gen fgg == 1 = primBox (fgsign fgg * b)

fgToBox :: Homology -> FreeGroup -> Box
fgToBox x (FG l) = foldr (&) unit (map (fggToBox x) l)
\end{code}

To see if we can short-circuit the evaluation, check at each stage if
all the boxes have become marked with a non-unique max or min.  If
this happens, then we can short-circuit evaluation: there is no way to
restore a unique max.

To give a more accurate picture, we don't short-circuit if there are
fewer than 7 branches in the interval exchange, since then the
interval exchange does not fill the surface and we are near the end
game.

(Note that without any guard on the length of boxes, the program will
get to the empty interval exchange and decide that all 0 of the remain
boxes are marked.)

\begin{code}
badBoxes :: IntervalExchange Box -> Bool
badBoxes ie = let boxes = map dat ie in
	      length boxes >= 7 && (all markedTop boxes || all markedBot boxes)

data Result = ResMultiCurve |
		ResNullHomology |
		ResBadBoxes { steps :: Int } |
		ResFiber { steps :: Int } |
		ResNoFiber { steps :: Int }
  deriving (Eq, Show, Read)

fibers :: Result -> Maybe Bool
fibers ResMultiCurve   = Nothing
fibers ResNullHomology = Nothing
fibers (ResBadBoxes _) = Just False
fibers (ResFiber _)    = Just True
fibers (ResNoFiber _)  = Just False

formatResult :: Result -> String
formatResult ResMultiCurve   = "Multiple curve"
formatResult ResNullHomology = "Null homologous curve"
formatResult (ResBadBoxes n) = "Attracting state after "++show n++" iterations"
formatResult (ResFiber n)    = "Fibers ("++show n++" iterations)"
formatResult (ResNoFiber n)  = "Does not fiber ("++show n++" iterations)"

doIE :: IntervalExchange FreeGroup -> Result
doIE ie = let hs = runIE (mapIE fgToHomology ie) in
	  case hs of
	    _:_:_           -> ResMultiCurve
	    [(n,_)] | n > 1 -> ResMultiCurve
	    [(1,(0,0))]     -> ResNullHomology
	    []		    -> ResNullHomology
	    [(1,h)] ->
	      case runIEstop badBoxes (mapIE (fgToBox (rotate90 h)) ie) of
		(n, Nothing) -> ResBadBoxes n
		(n, Just [(1,b)]) | marked b  -> ResNoFiber n
			          | otherwise -> ResFiber n
\end{code}

Copyright 2002, 2004, 2005 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
