\begin{code}
module Genus2Simplify where
import Genus2
import Genus2Moves
import DT
import PIL
import List(find, sort, minimumBy)
import Maybe(fromJust)
\end{code}

Twists on an annulus to make it look as simple as possible, in
half-twists.  naturalTwist' is always even (so some number of full
twists).

\begin{code}
naturalTwist, naturalTwist' :: DTAnnulus -> Integer
-- naturalTwist a = -floor (t a / m a + 1/2)
naturalTwist a | m a == 0 = 0
naturalTwist a = -((2*t a + m a)`div`(2*m a))
-- naturalTwist' a = -2*floor(t a/(2*m a) + 1/2)
naturalTwist' a | m a == 0 = 0
naturalTwist' a = -2*((t a + m a)`div`(2*m a))
\end{code}

simplify: apply second elementary transformations to make a curve seem
as simple as possible.  We only apply second elementary
transformations in order to preserve the 3-manifold and its Heegard
decomposition.

\begin{code}
simplePos :: Genus2 -> Genus2
simplePos g@(Barbell {}) = g
simplePos (Theta e1 e2 e3) =
   let [e1', e2', e3'] = sort [e1,e2,e3] in Theta e1' e3' e2'

complexity :: Genus2 -> (Coord, Coord)
complexity g = let g' = simplePos g in (m (e1 g') + m (e3 g'), m (e2 g'))

simpleMoves :: Genus2 -> [Move]
simpleMoves g@(Barbell {}) = [(2,naturalTwist (e2 g))]
simpleMoves g@(Theta {}) =
  [(1, naturalTwist (e1 g)),
   (2, naturalTwist (e2 g)),
   (3, naturalTwist (e3 g))]

simpleStep :: Genus2 -> Genus2
simpleStep g =
  let gs = map (\m -> move m g) (simpleMoves g)
  in minimumBy (\g1 g2 -> compare (complexity g1) (complexity g2)) gs

simplify :: Genus2 -> Genus2
simplify g = let g' = simpleStep g in
	     if complexity g' < complexity g then simplify g'
	     else g

goodBarbell :: Genus2 -> Bool
goodBarbell (Theta _ _ _) = False
goodBarbell (Barbell e1 e2 e3) = m e2 <= 2*min (m e1) (m e3)

curveRotations :: Genus2 -> [Genus2]
curveRotations (Theta e1 e2 e3) = [Theta e1 e2 e3,
				   Theta e2 e3 e1,
				   Theta e3 e1 e2]

standardForm :: Genus2 -> Genus2
standardForm g =
    let aux g@(Theta e1 e2 e3) = move (2,naturalTwist' e2) g
    in
        case g of
	 Barbell _ _ _ -> standardForm (simpleStep g)
	 Theta _ _ _ -> let t = find goodBarbell $ map aux $ curveRotations g
			in case t of
			    Just g -> g
			    Nothing -> error "standardForm: failed to simplify"
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
